@@ -48,6 +48,7 @@ import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.util.Arrays;
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
@@ -187,8 +188,8 @@ public class Monkey {
     /** Categories we are allowed to launch **/
     private ArrayList<String> mMainCategories = new ArrayList<String>();
 
-    /** Applications we can switch to. */
-    private ArrayList<ComponentName> mMainApps = new ArrayList<ComponentName>();
+    /** Applications we can switch to, as well as their corresponding categories. */
+    private HashMap<ComponentName, String> mMainApps = new HashMap<>();
 
     /** The delay between event inputs **/
     long mThrottle = 0;
@@ -549,6 +550,16 @@ public class Monkey {
         commandLineReport(bugreportName + ".txt", "bugreport");
     }
 
+    // UncaughtExceptionHandler set by RuntimeInit will report crash to system_server, which
+    // is not necessary for monkey and even causes deadlock. So we override it.
+    private static class KillSelfHandler implements Thread.UncaughtExceptionHandler {
+        @Override
+        public void uncaughtException(Thread t, Throwable e) {
+            Process.killProcess(Process.myPid());
+            System.exit(10);
+        }
+    }
+
     /**
      * Command-line entry point.
      *
@@ -558,6 +569,7 @@ public class Monkey {
         // Set the process name showing in "ps" or "top"
         Process.setArgV0("com.android.commands.monkey");
 
+        Thread.setDefaultUncaughtExceptionHandler(new KillSelfHandler());
         Logger.err.println("args: " + Arrays.toString(args));
         int resultCode = (new Monkey()).run(args);
         System.exit(resultCode);
@@ -1073,7 +1085,8 @@ public class Monkey {
                             Logger.out.println("//   + Using main activity " + r.activityInfo.name
                                     + " (from package " + packageName + ")");
                         }
-                        mMainApps.add(new ComponentName(packageName, r.activityInfo.name));
+                        mMainApps.put(
+                            new ComponentName(packageName, r.activityInfo.name), category);
                     } else {
                         if (mVerbose >= 3) { // very very verbose
                             Logger.out.println("//   - NOT USING main activity "
