@@ -68,10 +68,9 @@
 #include "incremental_server.h"
 #include "services.h"
 #include "shell_protocol.h"
+#include "socket_spec.h"
 #include "sysdeps/chrono.h"
 
-extern int gListenAll;
-
 DefaultStandardStreamsCallback DEFAULT_STANDARD_STREAMS_CALLBACK(nullptr, nullptr);
 
 static std::string product_file(const std::string& file) {
@@ -252,6 +251,9 @@ static void help() {
         " $ANDROID_LOG_TAGS        tags to be used by logcat (see logcat --help)\n"
         " $ADB_LOCAL_TRANSPORT_MAX_PORT max emulator scan port (default 5585, 16 emus)\n"
         " $ADB_MDNS_AUTO_CONNECT   comma-separated list of mdns services to allow auto-connect (default adb-tls-connect)\n"
+        "\n"
+        "Online documentation: https://android.googlesource.com/platform/packages/modules/adb/+/refs/heads/master/docs/user/adb.1.md\n"
+        "\n"
     );
     // clang-format on
 }
@@ -1232,7 +1234,9 @@ static int backup(int argc, const char** argv) {
 static int restore(int argc, const char** argv) {
     fprintf(stdout, "WARNING: adb restore is deprecated and may be removed in a future release\n");
 
-    if (argc != 2) error_exit("restore requires an argument");
+    if (argc < 2) {
+        error_exit("usage: adb restore FILENAME [ARG]...");
+    }
 
     const char* filename = argv[1];
     unique_fd tarFd(adb_open(filename, O_RDONLY));
@@ -1241,8 +1245,17 @@ static int restore(int argc, const char** argv) {
         return -1;
     }
 
+    std::string cmd = "restore:";
+    argc -= 2;
+    argv += 2;
+    while (argc-- > 0) {
+        cmd += " " + escape_arg(*argv++);
+    }
+
+    D("restore. filename=%s cmd=%s", filename, cmd.c_str());
+
     std::string error;
-    unique_fd fd(adb_connect("restore:", &error));
+    unique_fd fd(adb_connect(cmd, &error));
     if (fd < 0) {
         fprintf(stderr, "adb: unable to connect for restore: %s\n", error.c_str());
         return -1;
@@ -1544,6 +1557,7 @@ int adb_commandline(int argc, const char** argv) {
             if (isdigit(argv[0][2])) {
                 id = argv[0] + 2;
             } else {
+                if (argc < 2 || argv[0][2] != '\0') error_exit("-t requires an argument");
                 id = argv[1];
                 --argc;
                 ++argv;
@@ -1557,7 +1571,7 @@ int adb_commandline(int argc, const char** argv) {
         } else if (!strcmp(argv[0], "-e")) {
             transport_type = kTransportLocal;
         } else if (!strcmp(argv[0], "-a")) {
-            gListenAll = 1;
+            gListenAll = true;
         } else if (!strncmp(argv[0], "-H", 2)) {
             if (argv[0][2] == '\0') {
                 if (argc < 2) error_exit("-H requires an argument");
@@ -1932,7 +1946,10 @@ int adb_commandline(int argc, const char** argv) {
 
         parse_push_pull_args(&argv[1], argc - 1, &srcs, &dst, &copy_attrs, &sync, &compression,
                              &dry_run);
-        if (srcs.empty() || !dst) error_exit("push requires an argument");
+        if (srcs.empty() || !dst) {
+            error_exit("push requires <source> and <destination> arguments");
+        }
+
         return do_sync_push(srcs, dst, sync, compression, dry_run) ? 0 : 1;
     } else if (!strcmp(argv[0], "pull")) {
         bool copy_attrs = false;
