@@ -24,7 +24,7 @@
 
 // Layout of this struct must match struct WSABUF (verified via static assert in sysdeps_win32.cpp)
 struct adb_iovec {
-    size_t iov_len;
+    unsigned int iov_len;
     void* iov_base;
 };
 
