@@ -25,7 +25,7 @@ WindowToken.fromProto = function (proto: any, isActivityInTree: Boolean): Window
 
     const windowContainer = WindowContainer.fromProto(
         /* proto */ proto.windowContainer,
-        /* protoChildren */ proto.windowContainer.children.reverse(),
+        /* protoChildren */ proto.windowContainer?.children?.reverse() ?? [],
         /* isActivityInTree */ isActivityInTree,
         /* nameOverride */ null,
         /* identifierOverride */ null,
@@ -35,7 +35,6 @@ WindowToken.fromProto = function (proto: any, isActivityInTree: Boolean): Window
     entry.kind = entry.constructor.name;
     entry.proto = proto;
     entry.shortName = shortenName(entry.name);
-    console.warn("Created ", entry.kind, " stableId=", entry.stableId);
     return entry;
 }
 
